"""
Add index URL annotation
========================

This flag provides the ability to annotate the index URL mimicking the logic of
the ``pip-compile`` ``--index`` and ``--no-index` flag by opting to add or not
add the ``pip`` index to the generated files.

.. code-block:: text

    --annotate-index / --no-annotate-index    Add index URL to generated files (default false)

Note the default behavior is not to add the index, i.e., ``--no-annotate-index``.
"""

from .base import ClickOption
from .forward import ForwardOption


class AnnotateIndex(ForwardOption):
    """Optionally annotate the index URL to the generated files."""

    OPTION_NAME = "annotate_index"
    CLICK_OPTION = ClickOption(
        long_option="--annotate-index/--no-annotate-index",
        default=False,
        is_flag=True,
        help_text="Add the index URL to generated files (default false).",
    )
    enabled_pin_options = ["--index"]
    disabled_pin_options = ["--no-index"]



"""
Limit ``.in`` files
===================

By default ``pip-compile-multi`` compiles all ``.in`` files in ``requirements`` directory.
To limit compilation to only a subset, use

.. code-block:: text

    -n, --only-name TEXT        Compile only for passed environment names and
                                their references. Can be supplied multiple
                                times.

For example, to compile one file under Python2.7 and another under Python3.6, run:

.. code-block:: text

    $ virtual-env27/bin/pip-compile-multi -n deps27
    Locking requirements/deps27.in to requirements/deps27.txt. References: []
    $ virtual-env36/bin/pip-compile-multi -n deps36
    Locking requirements/deps36.in to requirements/deps36.txt. References: []


.. automodule:: pipcompilemulti.features.forbid_post
"""

from pipcompilemulti.utils import recursive_refs
from .base import BaseFeature, ClickOption


class LimitEnvs(BaseFeature):
    """Limit discovered environments to specified subset."""

    OPTION_NAME = 'include_names'
    CLICK_OPTION = ClickOption(
        long_option='--only-name',
        short_option='-n',
        multiple=True,
        help_text='Compile only for passed environment names and their '
                  'references. Can be supplied multiple times.',
    )

    def __init__(self):
        self._all_envs = None

    @property
    def direct_envs(self):
        """Set of environments included by command line options."""
        return set(self.value or [])

    def on_discover(self, env_confs):
        """Save set of all (recursive) included environments."""
        included_and_refs = self.direct_envs
        if not self.direct_envs:
            # No limit means all envs included:
            self._all_envs = [env['name'] for env in env_confs]
            return
        for name in set(included_and_refs):
            included_and_refs.update(
                recursive_refs(env_confs, name)
            )
        self._all_envs = included_and_refs

    def included(self, env_name):
        """Whether environment is included directly or by reference."""
        return env_name in self._all_envs



"""Remove packages included in referenced environments."""

import logging

from pipcompilemulti.utils import recursive_refs, merged_packages


logger = logging.getLogger("pip-compile-multi")


class PackageDeduplicator:
    """Remove packages included in referenced environments."""

    def __init__(self):
        self.env_packages = {}
        self.env_confs = None

    def on_discover(self, env_confs):
        """Save environment references."""
        self.env_confs = env_confs

    def register_packages_for_env(self, env_name, packages):
        """Save environment packages."""
        self.env_packages[env_name] = packages

    def ignored_packages(self, env_name):
        """Get package mapping from name to version for referenced environments."""
        if self.env_confs is None:
            return {}
        rrefs = recursive_refs(self.env_confs, env_name)
        return merged_packages(self.env_packages, rrefs)

    def recursive_refs(self, env_name):
        """Return recursive list of environment names referenced by env_name."""
        if self.env_confs is None:
            return {}
        return recursive_refs(self.env_confs, env_name)
